#!/usr/bin/php
<?php

/*
 * Author: Arsenij Chystyakov <achystyakov@n-ix.com.ua>
 *
 * Spider responsible for synchronization db and local or remote storage
 *
 * */

set_include_path('.:/opt/logserver/gui/libs/');
require_once ('config.php');

class Spider_Storage extends Config
{
    private $db_name = Config::dbname;
    private $db_pass = Config::dbpassword;
    private $db_user = Config::dbuser;
    private $db_host = Config::dbhost;
    private $db_port = "5432";

    private $path = array ("local" => "", "remote" => "");

    private $locmask = array ("local" => "101", "remote" => "110");

    private $uplocmask = array ("local" => "010", "remote" => "001");

    private $type = "";

    private $files_array = array();
    private $tables_registry_name = array();
    private $tables_registry_name_all = array();
    private $tables_registry_name_null = array();

    /* Constructor */

    public function __construct()
    {
        $type_array = array("local", "remote");
        $res = $this->connector();
        $this->get_config();
        foreach ($type_array as $type )
        {

            $loc = $this->path['local'];
            $rem = $this->path['remote'];

            if ($type == "local" && !is_dir($loc))
            {
                continue;
            }

            if (($rem == "" && $type == "remote") && !is_dir($rem))
            {
                exit();
            }

            $this->files_array = array();
            $this->tables_registry_name = array();
            $this->tables_registry_name_all = array();
            $this->tables_registry_name_null = array();

            $this->get_start_dir($type);
            $this->get_storage(".");
            $this->get_db_registry($type);
            $this->check_tables_registry($type);
        }
        $this->close_connector($res);
    }

    private function connector ()
    {
        $dbconn = pg_connect("dbname=$this->db_name user=$this->db_user");
        return $dbconn;
    }

    private function close_connector($res)
    {
        pg_close($res);
    }

    private function get_config()
    {
        $sql = "SELECT local_storage_path, remote_storage_path FROM settings";
        $result = pg_query($sql);

        if (pg_num_rows($result) == 0)
        {
            exit();
        }

        list($local, $remote) = pg_fetch_row($result);
        $this->path['local'] = $local;
        $this->path['remote'] = $remote;
    }

    private function get_start_dir ($type)
    {
        $path = $this->path[$type];
        chdir($path);
    }

    private function valid_file_name ($filename)
    {
        $result_filename = "messages_".str_replace("/", "_", substr($filename, 2));

    return $result_filename;
    }

    private function get_storage($path, $i=0)
    {
        $dir_handler = opendir($path);
        while(($files = readdir($dir_handler)) !== false)
        {
            if($files != "." && $files != "..")
            {
                if(is_file("$path/$files"))
                {

                   //preg_match("/[0-9]{1,2}\.bz2/", $files);
                   if((substr($files, -4) != ".bz2") || (substr($files, 0, 3) == "lf.") || (strlen($files) != 6)) continue;
                   $name = substr($files, 0, -4);
                   $this->files_array[] = $this->valid_file_name($path."_".$name);
                   $i++;
                }

                if(is_dir("$path/$files"))
                {
                   if (preg_match("/^[0-9]{4}\_[0-9]{2}$/", $files) || preg_match("/^[0-9]{2}$/", $files))
                        $this->get_storage("$path/$files");
                   if (count(scandir("$path/$files")) == 2)
                   {
                        rmdir("$path/$files");
                   }
                }
            }
        }
    }

    private function get_db_registry($type)
    {
        $sql = "select name FROM tables_registry WHERE (location & get_location_mask('$type'))::integer != 0 ORDER BY id";
        $result = pg_query($sql);

        while (list ($name) = pg_fetch_row($result))
        {
            $this->tables_registry_name[] = $name;
        }


        $mask = $this->locmask[$type];
        $sql = "select name, location FROM tables_registry WHERE (location | B'$mask')!= B'111' ORDER BY id";
        $result = pg_query($sql);

        while (list ($name) = pg_fetch_row($result))
        {
            $this->tables_registry_name_null[] = $name;
        }

        $sql = "select name FROM tables_registry ORDER BY id";
        $result = pg_query($sql);

        while (list ($name) = pg_fetch_row($result))
        {
            $this->tables_registry_name_all[] = $name;
        }

    }

    private function check_tables_registry($type)
    {
		/* make sure all tables have database bit in location set */
		pg_query("UPDATE tables_registry SET location = (location | B'100') WHERE name IN"
			. " (SELECT table_name FROM information_schema.tables WHERE table_name LIKE 'messages_%')");

		/* make sure unexistent tables do not have database bit set */
		pg_query("UPDATE tables_registry SET location = (location & B'011') WHERE name NOT IN"
			. " (SELECT table_name FROM information_schema.tables WHERE table_name LIKE 'messages_%')");

        /*********** Tables in Files *************/
        $mask = $this->locmask[$type];

        $tables_in_files = array_diff($this->tables_registry_name_all, $this->files_array);
        $tables_in_files_uncheck = array_diff($tables_in_files, $this->tables_registry_name_null);

        if (count($tables_in_files_uncheck) > 0)
        {
            $sql = "UPDATE tables_registry SET location = location & B'$mask' WHERE name IN (";

            foreach ($tables_in_files_uncheck as $name)
            {
                $sql .= "'$name', ";
            }

            $sql = substr($sql, 0, -2).");";

            $result = pg_query($sql);
        }

        $upmask = $this->uplocmask[$type];
        $update_tables_in_files = array_diff($this->files_array, $this->tables_registry_name);

        if (count($update_tables_in_files) > 0)
        {
            $sql = "UPDATE tables_registry SET location = location | B'$upmask' WHERE name IN (";

            foreach ($update_tables_in_files as $name)
            {
                $sql .= "'$name', ";
            }

            $sql = substr($sql, 0, -2).");";

            $result = pg_query($sql);
        }

        /*********** Files in Tables *************/

        $files_in_tables = array_diff($this->files_array, $this->tables_registry_name_all);

        if (count($files_in_tables) > 0)
        {

            if ($type == "local") $m = '010';
            if ($type == "remote") $m = '001';

            $sql = "INSERT INTO tables_registry (name, location, date) VALUES ";

            foreach ($files_in_tables as $name)
            {
                $sql .= "('$name', B'$m', current_timestamp), ";
            }

            $sql = substr($sql, 0, -2);

            $result = pg_query($sql);
        }
    }
}

$obj = new Spider_Storage();
?>
